% process the synhead files
% to create two matfiles containing
% SynBiwi+ (for partial domain adaptation, source dataset)
% Synhead++ (for domain adaptation, source dataset)
% tested with matlab 2019a on ubuntu 18
clear variables;

% from main file
global synhead_folder
root_dir = synhead_folder;

%% Synhead preparation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% collect backgrounds
bgs = dir(fullfile(root_dir,'background','*.jpg'));
backgrounds = cell(20,1);
if(length(bgs) ~=20)
  error('wrong folder?');
end

for i=1:20
  backgrounds{i} = fullfile(bgs(i).folder,bgs(i).name);
end

%% collect images
% subfolder scan
folders = dir(root_dir);

%%
subsets = {'BIWI','breitenstein','kinect','softkinetic'};
subjects = {'f01','f02','f03','f04','f05','m01','m02','m03','m04','m05'};

%%
all_images = cell(0);
subsetid_subjectid_folderid_frame_nr = [];
gt_poses = [];
counter = 1;
for subset_nr = 1:length(subsets)
  
  %% subset name
  name = subsets(subset_nr);
  fprintf('Processing subset: %s\n', name{1})
  subfol_csv = dir(fullfile(root_dir,subsets{subset_nr},'*.csv'));
  gt = cell(0);
  for subfolder_id = 1:length(subfol_csv)
    csv_name = [num2str(subfolder_id-1,'%02u') '.csv'];   
    gt{subfolder_id} = csvread(fullfile(subfol_csv(1).folder, csv_name));
  end
  %% read all csvs
  % go to all subject folders and get the individual sequences that belong
  % to the csv files
  
  % get subject
  for subject_id = 1:length(subjects)
    fprintf('Subject: %s\n',num2str(subject_id))
    for subfolder_id = 1:length(subfol_csv)
      folder_nr = subfolder_id-1;
      folder_name = num2str(folder_nr,'%02u');
      
      folder_tmp = fullfile(root_dir, subsets{subset_nr}, subjects{subject_id}, folder_name, '/*.png');
      images_dir = dir(folder_tmp);
      
      images = cell(0);
      % get the frames 
      temp = zeros(length(images_dir), 4);
      for frame=1:length(images_dir)
        temp(frame,:) = [subset_nr-1, subject_id-1, folder_nr, frame-1];
        images{frame} = fullfile(images_dir(frame).folder,images_dir(frame).name);
      end
      % append
      subsetid_subjectid_folderid_frame_nr = [subsetid_subjectid_folderid_frame_nr; temp];
      
      % check if number of images is the same as csv groundtruth
      if(size(gt{subfolder_id},1) ~= 0.5*(length(images)+length(images_dir)))
        error('image gt missmatch');
      end
      % append images            
      all_images = [all_images images];
      % append gt pose
      gt_poses = [gt_poses; gt{subfolder_id}];
      
    end
    
  end
end

%% 
images = all_images';
subjects = subjects';
subsets = subsets';
%% subject ids
ids = subsetid_subjectid_folderid_frame_nr(:,2);

%% change the poses (I used SpinCalc.m), we will just load correctives
%fprintf('Converting poses... \n');
%synhead_pose =  convert_euler_synhead(var_gt_pose);

%% load the pose correctives and bboxes for synhead+
plus_data = load(fullfile('mats','synhead_plus_data.mat'));
% change poses and store
plus_poses = gt_poses+plus_data.pose_correctives;
save('synhead_raw.mat','ids','gt_poses','plus_poses','images','subjects','subsets','backgrounds','-v7');
disp('done synhead_raw.');

%% crop data
images(plus_data.removed_images) = [];
ids(plus_data.removed_images) = [];
% check if images are in correct order
for i=1:length(images)
  if(strcmp(images{i}(end-19:end-4),plus_data.images{i}) == 0)
    error('error: order of images does not match.');
    % if images are not in correct order use plus_data.images 
  end
end
bboxes = plus_data.bboxes;
poses = plus_poses;
poses(plus_data.removed_images,:) = [];

%% save synhead plus
save('synhead_plus.mat','images','poses','bboxes','ids','subjects','subsets','backgrounds','-v7');
disp('done synhead_plus.');

%%
clear variables;
%% create synbiwi+ from synhead+
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
aug = load(fullfile('mats','synhead_aug_2_biwi.mat'));
synhead_raw = load('synhead_raw.mat');
global target_data_folder
global data_folder
target_dir = target_data_folder;

augs_synIdcs_flip_rot_images_pose = aug.augs_synIdcs_flip_rot_images_pose;
img_name_counter = 0;

synbiwi_poses  = zeros(156770,3);
check_poses  = zeros(156770,3);
synbiwi_bboxes = aug.bboxes;
synbiwi_images = cell(156770,1);
synbiwi_ids    = zeros(156770,1);

dry_run = 0;

for i=1:15677 % all biwi frames

  % augmentations
  flip = augs_synIdcs_flip_rot_images_pose{i,2};
  rotate = augs_synIdcs_flip_rot_images_pose{i,3};
  synhead_ids = augs_synIdcs_flip_rot_images_pose{i,1};
    
  for j=1:10 % we have 10 head models
    
    % load original synhead data
    img_file     = synhead_raw.images{synhead_ids(j)};
    new_synhead_pose = synhead_raw.plus_poses(synhead_ids(j),:);
    id = synhead_raw.ids(synhead_ids(j),:);
        
    % transform image
    if dry_run == 0
      [t_img,m,alpha] = imread(img_file);
      t_img(:,:,4) = alpha;
      
      new_img = rotate_image_z(t_img,-rotate,flip);
    end
    if flip == 1
      new_synhead_pose(2) = -new_synhead_pose(2);
      new_synhead_pose(3) = -new_synhead_pose(3);
    end
    
    new_synhead_pose(3) = new_synhead_pose(3)-rotate;
    
    % generate image name and update counter
    im_subname = [num2str(img_name_counter,'%06u') '.png'];
    fprintf([im_subname ' / 156770 \n']);
    img_name_counter = img_name_counter +1;
    %im_name = fullfile(target_dir, person_id, num2str(counter,'%06u'), '.png');
    im_name = fullfile(target_dir,im_subname);
    if dry_run == 0
      imwrite(new_img(:,:,1:3),im_name, 'Alpha',new_img(:,:,4));
    end
    %% save
    synbiwi_poses(img_name_counter,:) = new_synhead_pose;
    synbiwi_ids(img_name_counter) = id;
    synbiwi_images{img_name_counter} = im_name;
    
  end
end

%%
backgrounds = synhead_raw.backgrounds;
images = synbiwi_images;
poses  = synbiwi_poses;
bboxes = synbiwi_bboxes;
ids    = synbiwi_ids;
save(fullfile(data_folder,'synbiwi_plus.mat'),'images','poses','bboxes','ids','backgrounds','-v7');
disp('done synbiwi_plus.');
clearvars images poses bboxes ids;
%% add synbiwi+ to synhead+ to create synhead++
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
synbiwi_plus = load(fullfile(data_folder,'synbiwi_plus.mat'));
synhead_plus = load('synhead_plus.mat');
backgrounds     =  synhead_plus.backgrounds;
ids             = [synhead_plus.ids;    synbiwi_plus.ids];
poses           = [synhead_plus.poses;  synbiwi_plus.poses];
bboxes          = [synhead_plus.bboxes; synbiwi_plus.bboxes];
images          = [synhead_plus.images; synbiwi_plus.images]; 
%% save
save(fullfile(data_folder,'synhead_plus_plus.mat'), 'backgrounds','ids','poses','bboxes','images','-v7');
disp('done synhead_plus_plus.');
